﻿#include "our_service.h"
#include "our_spi.h"
#include "our_twi.h"
#include "our_fds.h"
#include "nrf_drv_spi.h"
#include "nrf_drv_twi.h"

#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include "string.h"
#include "nordic_common.h"
#include "app_error.h"
#include "app_uart.h"
#include "ble_db_discovery.h"
#include "app_timer.h"
#include "app_util.h"
#include "bsp_btn_ble.h"
#include "ble.h"
#include "ble_gap.h"
#include "ble_hci.h"
#include "nrf_sdh.h"
#include "nrf_sdh_ble.h"
#include "nrf_sdh_soc.h"
#include "ble_nus_c.h"
#include "nrf_ble_gatt.h"
#include "nrf_pwr_mgmt.h"
#include "nrf_ble_scan.h"

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#include "ble_conn_state.h"
#include "ble_conn_params.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "nrf_delay.h"
#include "nrf_ble_qwr.h"
#include "nrf_power.h"
#include "nrf_bootloader_info.h"
#include "ble_dfu.h"
#include "nrf_dfu_ble_svci_bond_sharing.h"
#include "nrf_svci_async_function.h"
#include "nrf_svci_async_handler.h"
#include "nrf_fstorage_sd.h"
#include "fds.h"

//adv
#define APP_ADV_INTERVAL                MSEC_TO_UNITS(100, UNIT_0_625_MS)                                            
#define APP_ADV_DURATION                0    
#define DEVICE_NAME                     "GT99"
#define MIN_CONNECTION_INTERVAL   MSEC_TO_UNITS(20, UNIT_1_25_MS)    
#define MAX_CONNECTION_INTERVAL   MSEC_TO_UNITS(75, UNIT_1_25_MS)     
#define SLAVE_LATENCY             0                                   
#define SUPERVISION_TIMEOUT       MSEC_TO_UNITS(500, UNIT_10_MS)     
static uint16_t   m_conn_handle = BLE_CONN_HANDLE_INVALID;
#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(5000)                       /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000)                      /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT    10   
static ble_uuid_t m_adv_uuids[] =                                               /**< Universally unique service identifiers. */
{
	 {BLE_UUID_OUR_SERVICE_UUID, BLE_UUID_TYPE_VENDOR_BEGIN}
};

typedef struct
{
	uint16_t file_id;
	uint16_t key;
	uint8_t data[12];
} fds_save_t;
#define LED 13
#define BTN 17

#define APP_BLE_CONN_CFG_TAG    1                                    
#define APP_BLE_OBSERVER_PRIO   3                                    

#define UART_TX_BUF_SIZE        256                                  
#define UART_RX_BUF_SIZE        256                                  


BLE_ADVERTISING_DEF(m_advertising);
NRF_BLE_QWR_DEF(m_qwr);
NRF_BLE_GATT_DEF(m_gatt);


APP_TIMER_DEF(m_one_sec_timer_id);
#define ONE_SEC_TIMER_INTERVAL                  APP_TIMER_TICKS(1000) // 1000 ms intervals

APP_TIMER_DEF(led_blink_timer_id);                                 //B2 repeat
#define LED_BLINK_TIMER_INTERVAL             APP_TIMER_TICKS(500)  //B2 r
APP_TIMER_DEF(ds3231_notification_timer_id);                       //C2 r
#define DS3231_NOTIFICATION_TIMER_INTERVAL   APP_TIMER_TICKS(1000) //C2 r
APP_TIMER_DEF(button_press_5s_timer_id);                           //button r
#define BUTTON_PRESS_5S_TIMER_INTERVAL       APP_TIMER_TICKS(10)   //button r


#define FLASH_END_ADDR 0xFFFFF
#define FLASH_STA_ADDR 0x0
uint32_t gNowFlashAddr;

bool record_state = false;
bool ds3231_notification = false;

ble_os_t m_our_service;
extern my_fds_info_t my_fds_info;
extern uint8_t our_record_interval_data[4];

const uint8_t model = 0x01;
const uint8_t version = 0x01;

static uint16_t m_ble_nus_max_data_len = BLE_GATT_ATT_MTU_DEFAULT - OPCODE_LENGTH - HANDLE_LENGTH;
uint8_t f249_data_array[31];

uint8_t  now_device_name[20];
uint8_t fds_device_name_concated[20];//device name into fds

uint32_t now_record_interval_in_sec = 15;

uint8_t default_pwd[9] = { 0xAA,0xA2,0x05,0x48,0x59,0x50,0x52,0x4f }; //HYPRO
uint8_t default_record_interval[5] = { 0xBB,0XB1,0x01,0x14,0x00 }; //20
uint8_t default_device_name[7] = { 0xCC,0xC3,0x04,0x47,0x54,0x30,0x36 };//GT06
bool gHaveRecordTime;
bool gHaveUartData;
uint8_t gSendDataStatus;


const uint8_t pwd_pass[8] = { 0x70,0x77,0x64,0x70,0x61,0x73,0x73,0x73 };
const uint8_t pwd_error[8] = { 0x70,0x77,0x64,0x65,0x72,0x72,0x6f,0x72 };

fds_save_t      pwd = { .file_id = 0xAB20, .key = 0xAB20, .data = "BBBBBBBBBBBB" };
fds_save_t      fds_pwd = { .file_id = 0x00AA, .key = 0x00A2, .data = "BBBBBBBBBBBB" };
fds_save_t      fds_record_interval = { .file_id = 0x0BB, .key = 0x00B1, .data = "BBBBBBBBBBBB" };
fds_save_t      fds_device_name = { .file_id = 0x00CC, .key = 0x00C3, .data = "BBBBBBBBBBBB" };
fds_save_t      fds_flash_addr = { .file_id = 0x00C0, .key = 0x00AA, .data = 0x00 };

uint32_t gF249_data;
const uint8_t monTable[12] = { 31,28,31,30,31,30,31,31,30,31,30,31 };

int16_t f249_decode(uint8_t hi, uint8_t low, uint8_t extra)
{
	int16_t value = 0;
	value += hi * 256 + low;

	int exponent = extra / 16;
	if (exponent == 0)
	{
		value = value * 100;
	}
	else if (exponent == 1)
	{
		value = value * 10;
	}
	else if (exponent == 2)
	{
		value = value * 1;
	}
	else if (exponent == 3)
	{
		value = value / 10;
	}
	int control = extra % 16;
	if (control % 2 != 0)
	{
		value = 0 - value;
	}
	return value;
}
static uint8_t check_year(uint16_t year)
{
	if (year % 4 == 0)
	{
		if (year % 100 == 0)
		{
			if (year % 400 == 0)
			{
				return 1;

			}
			else
			{
				return 0;

			}
		}
		else
		{
			return 1;
		}
	}
	else
	{
		return 0;
	}
}
uint32_t RTC_get_now(uint32_t timecount)
{
	static uint16_t daycnt = 0;
	uint32_t temp = 0;
	uint16_t temp1 = 0;
	uint32_t seccount = 0;
	uint16_t year;
	uint8_t month, date, hour, min, sec;
	seccount = timecount;
	temp = seccount / 86400;
	if (daycnt != temp)
	{
		daycnt = temp;
		temp1 = 1970;
		while (temp >= 365)
		{
			if (check_year(temp1))
			{
				if (temp >= 366)
				{
					temp -= 366;
				}
				else
				{
					temp1++;
					break;
				}
			}
			else
			{
				temp -= 365;
			}
			temp1++;
		}
		year = temp1;
		temp1 = 0;
		while (temp >= 28)
		{
			if (check_year(year) && temp1 == 1)
			{
				if (temp >= 29)
				{
					temp -= 29;
				}
				else
				{
					break;
				}
			}
			else
			{
				if (temp >= monTable[temp1])
				{
					temp -= monTable[temp1];
				}
				else
				{
					break;
				}
			}
			temp1++;
		}
		month = temp1 + 1;
		date = temp + 1;
	}
	temp = timecount % 86400;
	hour = temp / 3600;
	min = (temp % 3600) / 60;
	sec = (temp % 3600) % 60;
	//NRF_LOG_INFO("Time %d %d %d %d %d %d", year, month, date, hour, min, sec);

}
uint32_t day_time_set(uint8_t y, uint8_t mon, uint8_t day, uint8_t hour, uint8_t min, uint8_t sec)
{
	uint16_t t, year;
	uint32_t seccount = 0;
	//NRF_LOG_INFO("OR %d %d %d %d %d %d", y, mon, day, hour, min, sec);

	year = y + 2000;
	for (t = 1970; t < year; t++)
	{
		if (check_year(t))
		{
			seccount += 31622400;
		}
		else
		{
			seccount += 31536000;
		}
	}
	mon -= 1;
	for (t = 0; t < mon; t++)
	{
		seccount += (uint32_t)monTable[t] * 86400;

		if (check_year(year) && t == 1)
		{
			seccount += 86400;
		}
	}
	seccount += (uint32_t)(day - 1) * 86400;
	seccount += (uint32_t)hour * 3600;
	seccount += (uint32_t)min * 60;
	seccount += sec;

	NRF_LOG_INFO("Time %d", seccount);
	RTC_get_now(seccount);
	return seccount;
}
void assert_nrf_callback(uint16_t line_num, const uint8_t* p_file_name)
{
	app_error_handler(0xDEADBEEF, line_num, p_file_name);
}
static bool app_shutdown_handler(nrf_pwr_mgmt_evt_t event)
{
	switch (event)
	{
	case NRF_PWR_MGMT_EVT_PREPARE_DFU:
		NRF_LOG_INFO("Power management wants to reset to DFU mode.");
		break;
	default:
		return true;
	}
	NRF_LOG_INFO("Power management allowed to reset to DFU mode.");
	return true;
}
NRF_PWR_MGMT_HANDLER_REGISTER(app_shutdown_handler, 0);
static void buttonless_dfu_sdh_state_observer(nrf_sdh_state_evt_t state, void* p_context)
{
	if (state == NRF_SDH_EVT_STATE_DISABLED)
	{
		// Softdevice was disabled before going into reset. Inform bootloader to skip CRC on next boot.
		nrf_power_gpregret2_set(BOOTLOADER_DFU_SKIP_CRC);

		//Go to system off.
		nrf_pwr_mgmt_shutdown(NRF_PWR_MGMT_SHUTDOWN_GOTO_SYSOFF);
	}
}
NRF_SDH_STATE_OBSERVER(m_buttonless_dfu_state_obs, 0) =
{
	.handler = buttonless_dfu_sdh_state_observer,
};
static void advertising_config_get(ble_adv_modes_config_t* p_config)
{
	memset(p_config, 0, sizeof(ble_adv_modes_config_t));
	p_config->ble_adv_fast_enabled = true;
	p_config->ble_adv_fast_interval = APP_ADV_INTERVAL;
	p_config->ble_adv_fast_timeout = APP_ADV_DURATION;
}
static void disconnect(uint16_t conn_handle, void* p_context)
{
	UNUSED_PARAMETER(p_context);
	ret_code_t err_code = sd_ble_gap_disconnect(conn_handle, BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
	if (err_code != NRF_SUCCESS)
	{
		NRF_LOG_WARNING("Failed to disconnect connection. Connection handle: %d Error: %d", conn_handle, err_code);
	}
	else
	{
		NRF_LOG_DEBUG("Disconnected connection handle %d", conn_handle);
	}
}
static void ble_dfu_evt_handler(ble_dfu_buttonless_evt_type_t event)
{
	switch (event)
	{
	case BLE_DFU_EVT_BOOTLOADER_ENTER_PREPARE:
	{
		NRF_LOG_INFO("Device is preparing to enter bootloader mode.");
		ble_adv_modes_config_t config;
		advertising_config_get(&config);
		config.ble_adv_on_disconnect_disabled = true;
		ble_advertising_modes_config_set(&m_advertising, &config);
		uint32_t conn_count = ble_conn_state_for_each_connected(disconnect, NULL);
		NRF_LOG_INFO("Disconnected %d links.", conn_count);
		break;
	}
	case BLE_DFU_EVT_BOOTLOADER_ENTER:
		NRF_LOG_INFO("Device will enter bootloader mode.");
		break;
	case BLE_DFU_EVT_BOOTLOADER_ENTER_FAILED:
		NRF_LOG_ERROR("Request to enter bootloader mode failed asynchroneously.");
		break;
	case BLE_DFU_EVT_RESPONSE_SEND_ERROR:
		NRF_LOG_ERROR("Request to send a response to client failed.");
		APP_ERROR_CHECK(false);
		break;
	default:
		NRF_LOG_ERROR("Unknown event from ble_dfu_buttonless.");
		break;
	}
}
static void gap_params_init(void)
{
	uint32_t                err_code;
	ble_gap_conn_params_t   gap_conn_params;
	ble_gap_conn_sec_mode_t sec_mode;
	BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);
	if ((now_device_name[0] == 0xCC) && (now_device_name[1] == 0xC3))
	{
		NRF_LOG_INFO("customer device name\n");
		err_code = sd_ble_gap_device_name_set(&sec_mode,
			(const uint8_t*)(now_device_name + 3),
			now_device_name[2]);
	}
	else {
		NRF_LOG_INFO("default device name\n");
		err_code = sd_ble_gap_device_name_set(&sec_mode,
			(const uint8_t*)DEVICE_NAME,
			strlen(DEVICE_NAME));
	}

	APP_ERROR_CHECK(err_code);
	memset(&gap_conn_params, 0, sizeof(gap_conn_params));
	gap_conn_params.min_conn_interval = MIN_CONNECTION_INTERVAL;
	gap_conn_params.max_conn_interval = MAX_CONNECTION_INTERVAL;
	gap_conn_params.slave_latency = SLAVE_LATENCY;
	gap_conn_params.conn_sup_timeout = MSEC_TO_UNITS(4000, UNIT_10_MS);
	err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
	APP_ERROR_CHECK(err_code);
}
static void nrf_qwr_error_handler(uint32_t nrf_error)
{
	APP_ERROR_HANDLER(nrf_error);
}
static void services_init(void)
{
	uint32_t           err_code;
	nrf_ble_qwr_init_t qwr_init = { 0 };
	ble_dfu_buttonless_init_t dfus_init = { 0 };
	qwr_init.error_handler = nrf_qwr_error_handler;
	err_code = nrf_ble_qwr_init(&m_qwr, &qwr_init);
	APP_ERROR_CHECK(err_code);

	our_service_init(&m_our_service);
	dfus_init.evt_handler = ble_dfu_evt_handler;
	err_code = ble_dfu_buttonless_init(&dfus_init);
	APP_ERROR_CHECK(err_code);


}
static void on_conn_params_evt(ble_conn_params_evt_t* p_evt)
{
	uint32_t err_code;

	if (p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED)
	{
		err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
		APP_ERROR_CHECK(err_code);
	}
}
static void conn_params_error_handler(uint32_t nrf_error)
{
	APP_ERROR_HANDLER(nrf_error);
}
static void conn_params_init(void)
{
	uint32_t               err_code;
	ble_conn_params_init_t cp_init;

	memset(&cp_init, 0, sizeof(cp_init));

	cp_init.p_conn_params = NULL;
	cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
	cp_init.next_conn_params_update_delay = NEXT_CONN_PARAMS_UPDATE_DELAY;
	cp_init.max_conn_params_update_count = MAX_CONN_PARAMS_UPDATE_COUNT;
	cp_init.start_on_notify_cccd_handle = BLE_GATT_HANDLE_INVALID;
	cp_init.disconnect_on_fail = false;
	cp_init.evt_handler = on_conn_params_evt;
	cp_init.error_handler = conn_params_error_handler;

	err_code = ble_conn_params_init(&cp_init);
	APP_ERROR_CHECK(err_code);
}
static void on_adv_evt(ble_adv_evt_t ble_adv_evt)
{
	uint32_t err_code;
	switch (ble_adv_evt)
	{
	case BLE_ADV_EVT_FAST:
		break;
	case BLE_ADV_EVT_IDLE:
		break;
	default:
		break;
	}
}
static void advertising_init(void)
{
	ret_code_t             err_code;
	ble_advertising_init_t init;
	memset(&init, 0, sizeof(init));
	init.advdata.name_type = BLE_ADVDATA_FULL_NAME;
	init.advdata.include_appearance = false;
	init.advdata.include_ble_device_addr = true;
	init.advdata.flags = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;
	// 	init.advdata.uuids_complete.uuid_cnt = sizeof(m_adv_uuids) / sizeof(m_adv_uuids[0]);
	// 	init.advdata.uuids_complete.p_uuids = m_adv_uuids;
	init.config.ble_adv_fast_enabled = true;
	init.config.ble_adv_fast_interval = APP_ADV_INTERVAL;
	init.config.ble_adv_fast_timeout = APP_ADV_DURATION;
	init.evt_handler = on_adv_evt;
	err_code = ble_advertising_init(&m_advertising, &init);
	APP_ERROR_CHECK(err_code);
	ble_advertising_conn_cfg_tag_set(&m_advertising, APP_BLE_CONN_CFG_TAG);
}
static void advertising_start(void)
{
	uint32_t err_code = ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST);

}
static uint32_t check_flash_num(void)
{
	uint8_t tmp[32];
	uint32_t flashNum;
	for (uint32_t i = FLASH_STA_ADDR; i < FLASH_END_ADDR; ++i)
	{
		W25Q80_Flash_BufferRead(tmp, i, 4);
		if (tmp[0] == 0xFF && tmp[1] == 0xFF && tmp[2] == 0xFF && tmp[3] == 0xFF)
		{
			flashNum = FLASH_END_ADDR - i;
			break;
		}
	}
	return flashNum / 8;
}
void uart_event_handle(app_uart_evt_t* p_event)
{
	static uint8_t data_array[BLE_NUS_MAX_DATA_LEN];
	static uint16_t index = 0;
	uint32_t ret_val;

	switch (p_event->evt_type)
	{
		/**@snippet [Handling data from UART] */
	case APP_UART_DATA_READY:
		UNUSED_VARIABLE(app_uart_get(&data_array[index]));
		index++;

		if ((data_array[index - 2] == 0x0D) || (data_array[index - 1] == 0x0A))
		{
			if (index == 31)
			{
				if (!gHaveUartData && gSendDataStatus == 0)
				{
					NRF_LOG_INFO("UART get F249 31 byte in APP_UART_DATA case\n");
					for (int i = 0; i < 31; i++)
					{
						f249_data_array[i] = data_array[i];
					}
					NRF_LOG_INFO("In data notify handler notify f249 data\n");
					int16_t f249_humidity = f249_decode(f249_data_array[2], f249_data_array[3], f249_data_array[4]);
					int16_t f249_temperature = f249_decode(f249_data_array[5], f249_data_array[6], f249_data_array[7]);
					gF249_data = f249_humidity << 16 | f249_temperature;
					NRF_LOG_INFO("f249_humidity = %d, f249_temperature = %d\n", f249_humidity, f249_temperature);
					gHaveUartData = true;
					gSendDataStatus = 2;
				}
			}
			else if (index > 1) {
				NRF_LOG_INFO("---------------maybe uart data lose------\n");
			}
			index = 0;
		}
		break;

		/**@snippet [Handling data from UART] */
	case APP_UART_COMMUNICATION_ERROR:
		NRF_LOG_ERROR("Communication error occurred while handling UART.");
		break;

	case APP_UART_FIFO_ERROR:
		NRF_LOG_ERROR("Error occurred in FIFO module used by UART.");
		break;

	default:
		break;
	}
}
static bool shutdown_handler(nrf_pwr_mgmt_evt_t event)
{
	ret_code_t err_code;

	err_code = bsp_indication_set(BSP_INDICATE_IDLE);
	APP_ERROR_CHECK(err_code);

	switch (event)
	{
	case NRF_PWR_MGMT_EVT_PREPARE_WAKEUP:
		// Prepare wakeup buttons.
		err_code = bsp_btn_ble_sleep_mode_prepare();
		APP_ERROR_CHECK(err_code);
		break;

	default:
		break;
	}

	return true;
}
NRF_PWR_MGMT_HANDLER_REGISTER(shutdown_handler, APP_SHUTDOWN_HANDLER_PRIORITY);
static void ble_evt_handler(ble_evt_t const* p_ble_evt, void* p_context)
{
	ret_code_t            err_code;
	ble_gap_evt_t const* p_gap_evt = &p_ble_evt->evt.gap_evt;

	switch (p_ble_evt->header.evt_id)
	{
	case BLE_GAP_EVT_CONNECTED:

		NRF_LOG_INFO("Connected");

		m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
		err_code = nrf_ble_qwr_conn_handle_assign(&m_qwr, m_conn_handle);
		APP_ERROR_CHECK(err_code);


		break;

	case BLE_GAP_EVT_DISCONNECTED:

		NRF_LOG_INFO("Disconnected");
		// LED indication will be changed when advertising starts.
		m_conn_handle = BLE_CONN_HANDLE_INVALID;


		break;

	case BLE_GAP_EVT_TIMEOUT:
		if (p_gap_evt->params.timeout.src == BLE_GAP_TIMEOUT_SRC_CONN)
		{
			NRF_LOG_INFO("Connection Request timed out.");
		}
		break;
	case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
		// Pairing not supported.
		err_code = sd_ble_gap_sec_params_reply(p_ble_evt->evt.gap_evt.conn_handle, BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP, NULL, NULL);
		APP_ERROR_CHECK(err_code);
		break;

	case BLE_GAP_EVT_CONN_PARAM_UPDATE_REQUEST:
		// Accepting parameters requested by peer.
		err_code = sd_ble_gap_conn_param_update(p_gap_evt->conn_handle,
			&p_gap_evt->params.conn_param_update_request.conn_params);
		APP_ERROR_CHECK(err_code);
		break;

	case BLE_GAP_EVT_PHY_UPDATE_REQUEST:
	{
		NRF_LOG_DEBUG("PHY update request.");
		ble_gap_phys_t const phys =
		{
			.rx_phys = BLE_GAP_PHY_AUTO,
			.tx_phys = BLE_GAP_PHY_AUTO,
		};
		err_code = sd_ble_gap_phy_update(p_ble_evt->evt.gap_evt.conn_handle, &phys);
		APP_ERROR_CHECK(err_code);
	} break;

	case BLE_GATTC_EVT_TIMEOUT:
		// Disconnect on GATT Client timeout event.
		NRF_LOG_DEBUG("GATT Client Timeout.");
		err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gattc_evt.conn_handle,
			BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
		APP_ERROR_CHECK(err_code);
		break;

	case BLE_GATTS_EVT_TIMEOUT:
		// Disconnect on GATT Server timeout event.
		NRF_LOG_DEBUG("GATT Server Timeout.");
		err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gatts_evt.conn_handle,
			BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
		APP_ERROR_CHECK(err_code);
		break;
	default:
		break;
	}
}
static void ble_stack_init(void)
{
	ret_code_t err_code;

	err_code = nrf_sdh_enable_request();
	APP_ERROR_CHECK(err_code);

	// Configure the BLE stack using the default settings.
	// Fetch the start address of the application RAM.
	uint32_t ram_start = 0;
	err_code = nrf_sdh_ble_default_cfg_set(APP_BLE_CONN_CFG_TAG, &ram_start);
	APP_ERROR_CHECK(err_code);

	// Enable BLE stack.
	err_code = nrf_sdh_ble_enable(&ram_start);
	APP_ERROR_CHECK(err_code);

	// Register a handler for BLE events.
	NRF_SDH_BLE_OBSERVER(m_ble_observer, APP_BLE_OBSERVER_PRIO, ble_evt_handler, NULL);
	NRF_SDH_BLE_OBSERVER(m_our_service_observer, APP_BLE_OBSERVER_PRIO, ble_our_service_on_ble_evt, (void*)&m_our_service);

}
void gatt_evt_handler(nrf_ble_gatt_t* p_gatt, nrf_ble_gatt_evt_t const* p_evt)
{
	if (p_evt->evt_id == NRF_BLE_GATT_EVT_ATT_MTU_UPDATED)
	{
		m_ble_nus_max_data_len = p_evt->params.att_mtu_effective - OPCODE_LENGTH - HANDLE_LENGTH;
	}
}
void gatt_init(void)
{
	ret_code_t err_code;

	err_code = nrf_ble_gatt_init(&m_gatt, gatt_evt_handler);
	APP_ERROR_CHECK(err_code);

	err_code = nrf_ble_gatt_att_mtu_central_set(&m_gatt, NRF_SDH_BLE_GATT_MAX_MTU_SIZE);
	APP_ERROR_CHECK(err_code);
}
void bsp_event_handler(bsp_event_t event)
{
	ret_code_t err_code;

	switch (event)
	{
	case BSP_EVENT_SLEEP:
		nrf_pwr_mgmt_shutdown(NRF_PWR_MGMT_SHUTDOWN_GOTO_SYSOFF);
		break;

	case BSP_EVENT_DISCONNECT:
		NRF_LOG_INFO("-------BSP_EVENT_DISCONNECT------\n");
		err_code = sd_ble_gap_disconnect(m_conn_handle,
			BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
		if (err_code != NRF_ERROR_INVALID_STATE)
		{
			APP_ERROR_CHECK(err_code);
		}
		advertising_start();
		break; // BSP_EVENT_DISCONNECT

	default:
		break;
	}
}
static void uart_init(void)
{
	ret_code_t err_code;

	app_uart_comm_params_t const comm_params =
	{
		.rx_pin_no = 11,
		.flow_control = APP_UART_FLOW_CONTROL_DISABLED,
		.use_parity = false,
		.baud_rate = UART_BAUDRATE_BAUDRATE_Baud9600
	};

	APP_UART_FIFO_INIT(&comm_params,
		UART_RX_BUF_SIZE,
		UART_TX_BUF_SIZE,
		uart_event_handle,
		APP_IRQ_PRIORITY_LOWEST,
		err_code);

	APP_ERROR_CHECK(err_code);
}
static void test_oneshot(void)
{
	ret_code_t rc;
	fds_record_desc_t desc = { 0 };
	fds_find_token_t  tok = { 0 };
	uint8_t* data;

	if (test_oneshot_event == 0xE1)
	{
		uint16_t write_file_id = 0xAB22;
		uint16_t write_record_key = 0xAB22;
		uint8_t write_data[] = "GT06_6666666";

		memset(&tok, 0x00, sizeof(fds_find_token_t));

		rc = fds_record_find(write_file_id, write_record_key, &desc, &tok);
		if (rc == FDS_SUCCESS)
		{
			fds_flash_record_t temp = { 0 };
			rc = fds_record_open(&desc, &temp);
			APP_ERROR_CHECK(rc);
			data = (uint8_t*)temp.p_data;
			for (uint8_t i = 0; i < temp.p_header->length_words * 4; i++)
			{
				NRF_LOG_INFO("%x / ", data[i]);
			}
			rc = fds_record_close(&desc);
			APP_ERROR_CHECK(rc);
			kls_fds_update(write_file_id, write_record_key, write_data);
		}
		else
		{
			NRF_LOG_INFO("EEE1");
			kls_fds_write(write_file_id, write_record_key, write_data);
		}
	}
	else if (test_oneshot_event == 0xE2)
	{
		uint16_t write_file_id = 0xAB22;
		uint16_t write_record_key = 0xAB22;
		uint8_t write_data[] = "GT06_6666";
		memset(&tok, 0x00, sizeof(fds_find_token_t));
		rc = fds_record_find(write_file_id, write_record_key, &desc, &tok);
		if (rc == FDS_SUCCESS)
		{
			fds_flash_record_t temp = { 0 };
			rc = fds_record_open(&desc, &temp);
			APP_ERROR_CHECK(rc);
			data = (uint8_t*)temp.p_data;
			for (uint8_t i = 0; i < temp.p_header->length_words * 4; i++)
			{
				NRF_LOG_INFO("%x / ", data[i]);
			}
			rc = fds_record_close(&desc);
			APP_ERROR_CHECK(rc);
		}
	}
}
static void one_sec_timer_handler(void* p_context)
{
	static uint8_t startCnt = 0;
	uint32_t tmpNum;

	if (record_state)
	{
		if (startCnt++ >= now_record_interval_in_sec)
		{
			startCnt = 0;
			gHaveRecordTime = true;
		}
	}
	else
	{
		gHaveRecordTime = false;
	}
	tmpNum = check_flash_num();
	update_attribute(&m_our_service, model, version, record_state, our_record_interval_data, tmpNum);
	NRF_LOG_INFO("FLASH NUM is %X\n", tmpNum);

	//	ID = W25Q80_ReadID();
	// 	if (ID != FLASH_ID)
	// 	{
	// 		NRF_LOG_INFO("init w25q80 error\r\n");
	// 		NRF_LOG_INFO("FLASH ID is %X\n", ID);
	// 	}
	// 	else
	// 	{
	// 		NRF_LOG_INFO("init w25q80 ok!\r\n");
	// 		NRF_LOG_INFO("FLASH ID is %X\n", ID);
	// 	}
}

static void led_blink_timer_handler(void* p_context) //B2
{
	nrf_gpio_pin_toggle(LED);
}
static void button_press_5s_timer_handler(void* p_context)
{
	static uint16_t button_press_counter = 0;
	bool nowStatus = nrf_gpio_pin_read(BTN);
	if (!nowStatus)
	{
		if (button_press_counter <= 500)
		{
			button_press_counter++;
			if (button_press_counter == 500)
			{
				NRF_LOG_INFO("you press 5 sec.........................................\n");

				if (record_state == false)
				{
					record_state = true;
					app_timer_start(led_blink_timer_id, LED_BLINK_TIMER_INTERVAL, NULL);
				}
				else if (record_state == true)
				{
					record_state = false;
					app_timer_stop(led_blink_timer_id);
					nrf_gpio_pin_clear(LED);
					//and send all data to phone
				}

				//start record
			}
		}
	}
	else
	{
		button_press_counter = 0;
	}


}
static void ds3231_notification_timer_handler(void* p_context) //C2
{
	DS3231_read();
	day_time_set(date_time[0], date_time[1], date_time[2], date_time[3], date_time[4], date_time[5]);
	NRF_LOG_INFO("In ds3231_notification_timer_handler\n");
	if (m_our_service.conn_handle != BLE_CONN_HANDLE_INVALID)
	{

		uint16_t               len = 6;
		ble_gatts_hvx_params_t hvx_params;
		memset(&hvx_params, 0, sizeof(hvx_params));

		hvx_params.handle = m_our_service.other_char_handles.value_handle;
		hvx_params.type = BLE_GATT_HVX_NOTIFICATION;
		hvx_params.offset = 0;
		hvx_params.p_len = &len;
		hvx_params.p_data = (uint8_t*)date_time;

		sd_ble_gatts_hvx(m_our_service.conn_handle, &hvx_params);

	}
}
static void confirm_password(void) //A1
{
	ret_code_t rc;
	fds_record_desc_t desc = { 0 };
	fds_find_token_t  tok = { 0 };
	uint8_t* pwd_data;
	bool result = false;
	NRF_LOG_INFO("password from ble: ");
	for (int i = 0; i < our_confirm_password[2]; i++)
	{
		NRF_LOG_INFO("%x ", our_confirm_password[i + 3]);
	}

	NRF_LOG_INFO("Start searching confirm_password...\n");
	memset(&tok, 0x00, sizeof(fds_find_token_t));

	rc = fds_record_find(fds_pwd.file_id, fds_pwd.key, &desc, &tok);
	if (rc != FDS_SUCCESS)
	{
		NRF_LOG_INFO("no pwd in fds...\n");
	}
	else
	{
		fds_flash_record_t temp = { 0 };
		rc = fds_record_open(&desc, &temp);
		NRF_LOG_INFO("Found Record ID = %d\n", desc.record_id);
		pwd_data = (uint8_t*)temp.p_data;
		NRF_LOG_INFO("password in fds: ");
		for (uint8_t i = 0; i < temp.p_header->length_words * 4; i++)
		{
			NRF_LOG_INFO("%x / ", pwd_data[i]);
		}
		if (pwd_data[0] == 0xAA && pwd_data[1] == 0xA2 && pwd_data[2] == (our_confirm_password[2]))
		{
			int error_count = 0;
			for (int i = 3; i < our_confirm_password[2]; i++)
			{
				if (our_confirm_password[i] != pwd_data[i])
				{
					error_count++;
				}
			}
			if (error_count == 0) {
				NRF_LOG_INFO("result: right pwd\n");
				result = true;
			}
			else
			{
				NRF_LOG_INFO("result: wrong pwd - 01\n");
				result = false;
			}
		}
		else
		{
			NRF_LOG_INFO("length_words in fds: %d , (our_confirm_password[2]): %d\n", pwd_data[2], (our_confirm_password[2]));
			NRF_LOG_INFO("result: wrong pwd - 02\n");
			result = false;
		}
		rc = fds_record_close(&desc);
		APP_ERROR_CHECK(rc);
	}

	if (m_our_service.conn_handle != BLE_CONN_HANDLE_INVALID) {
		uint16_t               len = 8;
		ble_gatts_hvx_params_t hvx_params;
		memset(&hvx_params, 0, sizeof(hvx_params));

		hvx_params.handle = m_our_service.pwd_char_handles.value_handle;
		hvx_params.type = BLE_GATT_HVX_NOTIFICATION;
		hvx_params.offset = 0;
		hvx_params.p_len = &len;
		if (result)
		{
			hvx_params.p_data = (uint8_t*)pwd_pass;  //notify a word express right password
		}
		else
		{
			hvx_params.p_data = (uint8_t*)pwd_error; //notify a word express wrong password
		}

		sd_ble_gatts_hvx(m_our_service.conn_handle, &hvx_params);
	}

}
static void change_password(void) //A2
{
	kls_fds_update(fds_pwd.file_id, fds_pwd.key, our_change_password);
	if (m_our_service.conn_handle != BLE_CONN_HANDLE_INVALID)
	{
		uint16_t               len = 7;
		ble_gatts_hvx_params_t hvx_params;
		memset(&hvx_params, 0, sizeof(hvx_params));

		hvx_params.handle = m_our_service.pwd_char_handles.value_handle;
		hvx_params.type = BLE_GATT_HVX_NOTIFICATION;
		hvx_params.offset = 0;
		hvx_params.p_len = &len;
		hvx_params.p_data = (uint8_t*)pwd_pass;

		sd_ble_gatts_hvx(m_our_service.conn_handle, &hvx_params);
	}
}
static void set_record_interval(void) //B1
{
	if (our_record_interval_data[2] == 0x01)
	{
		now_record_interval_in_sec = 0;
		now_record_interval_in_sec = our_record_interval_data[3];
		NRF_LOG_INFO("set_record_interval = %d\n", now_record_interval_in_sec);
		NRF_LOG_INFO("set_record_interval: %d\n", our_record_interval_data[3]);
		kls_fds_update(fds_record_interval.file_id, fds_record_interval.key, our_record_interval_data);
		NRF_LOG_INFO("set_record_interval done\n");
	}
	else if (our_record_interval_data[2] == 0x02)
	{
		now_record_interval_in_sec = 0;
		now_record_interval_in_sec = ((our_record_interval_data[3] << 8) + our_record_interval_data[4]) * 60;
		NRF_LOG_INFO("set_record_interval = %d\n", now_record_interval_in_sec);
		kls_fds_update(fds_record_interval.file_id, fds_record_interval.key, our_record_interval_data);
		NRF_LOG_INFO("set_record_interval done\n");
	}
	else
	{
		NRF_LOG_INFO("set_record_interval data wrong\n");
	}
}

static void change_device_name(void) //C3
{
	uint8_t str1[] = "GT06_";
	int len = strlen(str1) + our_device_name_data[2];// + 1;
	int fds_len = strlen(str1) + our_device_name_data[2] + 3;// + 1;
	uint8_t concated[len];

	memset(concated, '\0', len);
	strcat(concated, str1);
	strcat(concated, our_device_name_data + 3);
	NRF_LOG_INFO("len = %d ,name = %s\n", len, concated);

	memset(fds_device_name_concated, '\0', 20);
	fds_device_name_concated[0] = 0xCC;
	fds_device_name_concated[1] = 0xC3;
	fds_device_name_concated[2] = len;
	strcat(fds_device_name_concated, concated);
	NRF_LOG_INFO("fds_len = %d ,fds_device_name_concated = %s\n", fds_len, fds_device_name_concated);

	NRF_LOG_INFO("update device name\n");
	kls_fds_update(fds_device_name.file_id, fds_device_name.key, fds_device_name_concated);
	NRF_LOG_INFO("update done\n");

	ble_gap_conn_sec_mode_t sec_mode;
	BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);
	sd_ble_gap_device_name_set(&sec_mode, (const uint8_t*)concated, len);

	uint16_t m_adv_current_slave_link_conn_handle_saved = m_advertising.current_slave_link_conn_handle;
	advertising_init();
	m_advertising.current_slave_link_conn_handle = m_adv_current_slave_link_conn_handle_saved;

}
static void send_data_to_phone(void)
{
	uint8_t tmp[64];
	uint32_t timeCnt;
	uint8_t* flashTime = (uint8_t*)&timeCnt;
	uint8_t* flashData = (uint8_t*)&gF249_data;
	ret_code_t                 err_code;
	uint16_t               len;
	ble_gatts_hvx_params_t hvx_params;

	switch (gSendDataStatus)
	{
	case 1:
		for (uint32_t i = FLASH_STA_ADDR; i < FLASH_END_ADDR; i += 8)
		{
			W25Q80_Flash_BufferRead(tmp, i, 8);
			if (tmp[0] == 0xFF && tmp[1] == 0xFF && tmp[2] == 0xFF && tmp[3] == 0xFF)
			{

				break;
			}
			else
			{
				if (m_our_service.conn_handle != BLE_CONN_HANDLE_INVALID)
				{
					do
					{
						len = 8;
						memset(&hvx_params, 0, sizeof(hvx_params));
						hvx_params.handle = m_our_service.record_char_handles.value_handle;
						hvx_params.type = BLE_GATT_HVX_NOTIFICATION;
						hvx_params.offset = 0;
						hvx_params.p_len = &len;
						hvx_params.p_data = (uint8_t*)tmp;
						err_code = sd_ble_gatts_hvx(m_our_service.conn_handle, &hvx_params);
						if ((err_code != NRF_ERROR_INVALID_STATE) &&
							(err_code != NRF_ERROR_RESOURCES) &&
							(err_code != NRF_ERROR_NOT_FOUND))
						{
							APP_ERROR_CHECK(err_code);
						}						
					} while (err_code == NRF_ERROR_RESOURCES);
				}
			}
		}
		W25Q80_Chip_Erase();
		gNowFlashAddr = FLASH_STA_ADDR;
		break;
	case 2:
		if (m_our_service.conn_handle != BLE_CONN_HANDLE_INVALID)
		{
			uint16_t               len = 4;
			ble_gatts_hvx_params_t hvx_params;
			memset(&hvx_params, 0, sizeof(hvx_params));
			hvx_params.handle = m_our_service.char_handles.value_handle;
			hvx_params.type = BLE_GATT_HVX_NOTIFICATION;
			hvx_params.offset = 0;
			hvx_params.p_len = &len;
			hvx_params.p_data = (uint8_t*)&gF249_data;
			sd_ble_gatts_hvx(m_our_service.conn_handle, &hvx_params);
		}
		if (gHaveRecordTime)
		{
			DS3231_read();
			timeCnt = day_time_set(date_time[0], date_time[1], date_time[2], date_time[3], date_time[4], date_time[5]);
			if (gNowFlashAddr + 0x08 < FLASH_END_ADDR)
			{
				W25Q80_FLASH_PageWrite(flashTime, gNowFlashAddr, 4);
				gNowFlashAddr += 0x04;
				W25Q80_FLASH_PageWrite(flashData, gNowFlashAddr, 4);
				gNowFlashAddr += 0x04;
			}
			gHaveRecordTime = false;
		}
		gHaveUartData = false;
		break;
	}
	gSendDataStatus = 0;

}

static void process(void)
{
	switch (our_ble_event)
	{
	case 0xEE:
		our_ble_event = 0;
		NRF_LOG_INFO("test_oneshot\n");
		test_oneshot();
		break;
	case 0xA1:
		our_ble_event = 0;
		NRF_LOG_INFO("to confirm password\n");
		confirm_password();
		break;
	case 0xA2:
		our_ble_event = 0;
		NRF_LOG_INFO("to change password\n");
		change_password();
		break;
	case 0xB1:
		our_ble_event = 0;
		NRF_LOG_INFO("In ble_event_scan_timer_handler : set interval\n");
		set_record_interval();
		break;
	case 0xB2:
		our_ble_event = 0;
		if (record_state == false)
		{
			record_state = true;
			app_timer_start(led_blink_timer_id, LED_BLINK_TIMER_INTERVAL, NULL);
		}
		else
		{
			record_state = false;
			app_timer_stop(led_blink_timer_id);
		}
		break;
	case 0xB3:
		our_ble_event = 0;
		if (record_state == true)
		{
			record_state = false;
			app_timer_stop(led_blink_timer_id);
		}
		gSendDataStatus = 1;
		break;
	case 0xC2:
		our_ble_event = 0;
		if (ds3231_notification)
		{
			ds3231_notification = false;
			NRF_LOG_INFO("stop ds3231 notify\n");
			app_timer_stop(ds3231_notification_timer_id);
		}
		else
		{
			ds3231_notification = true;
			NRF_LOG_INFO("start ds3231 notify\n");
			app_timer_start(ds3231_notification_timer_id, DS3231_NOTIFICATION_TIMER_INTERVAL, NULL);
		}
		break;
	case 0xC3:
		our_ble_event = 0;
		NRF_LOG_INFO("In ble_event device namer change\n");
		change_device_name();
		break;

	default:
		// No implementation needed.
		break;
	}
}
static void timer_init(void)
{
	ret_code_t err_code = app_timer_init();
	app_timer_create(&m_one_sec_timer_id, APP_TIMER_MODE_REPEATED, one_sec_timer_handler);
	app_timer_create(&led_blink_timer_id, APP_TIMER_MODE_REPEATED, led_blink_timer_handler);                       //B2
	app_timer_create(&ds3231_notification_timer_id, APP_TIMER_MODE_REPEATED, ds3231_notification_timer_handler);   //C2
	app_timer_create(&button_press_5s_timer_id, APP_TIMER_MODE_REPEATED, button_press_5s_timer_handler);           //button
	APP_ERROR_CHECK(err_code);
}
static void log_init(void)
{
	ret_code_t err_code = NRF_LOG_INIT(NULL);
	APP_ERROR_CHECK(err_code);

	NRF_LOG_DEFAULT_BACKENDS_INIT();
}
static void power_management_init(void)
{
	ret_code_t err_code;
	err_code = nrf_pwr_mgmt_init();
	APP_ERROR_CHECK(err_code);
}
static void idle_state_handle(void)
{
	if (NRF_LOG_PROCESS() == false)
	{
		nrf_pwr_mgmt_run();
	}
}
static void pwd_init(void)
{

	ret_code_t rc;
	fds_record_desc_t desc = { 0 };
	fds_find_token_t  tok = { 0 };
	uint8_t* pwd_data;

	NRF_LOG_INFO("\npwd_init\n");
	memset(&tok, 0x00, sizeof(fds_find_token_t));

	rc = fds_record_find(fds_pwd.file_id, fds_pwd.key, &desc, &tok);
	if (rc != FDS_SUCCESS)
	{
		kls_fds_write(fds_pwd.file_id, fds_pwd.key, default_pwd);
	}
	else
	{
		fds_flash_record_t temp = { 0 };
		rc = fds_record_open(&desc, &temp);
		APP_ERROR_CHECK(rc);
		pwd_data = (uint8_t*)temp.p_data;
		if (pwd_data[0] == 0xAA && pwd_data[1] == 0xA2)
		{
			NRF_LOG_INFO("pwd ini AA A2 checked\n");
		}
		else
		{
			NRF_LOG_INFO("pwd ini error no AA A1\n");
		}
		rc = fds_record_close(&desc);
		APP_ERROR_CHECK(rc);
	}
}
static void device_name_init(void)
{
	ret_code_t rc;
	fds_record_desc_t desc = { 0 };
	fds_find_token_t  tok = { 0 };
	uint8_t* name_data;

	NRF_LOG_INFO("\ndevice_name_init");
	memset(&tok, 0x00, sizeof(fds_find_token_t));

	rc = fds_record_find(fds_device_name.file_id, fds_device_name.key, &desc, &tok);
	if (rc != FDS_SUCCESS)
	{
		kls_fds_write(fds_device_name.file_id, fds_device_name.key, default_device_name);
		memset(now_device_name, '\0', 20);
		memcpy(now_device_name, default_device_name, sizeof(default_device_name));
	}
	else
	{
		fds_flash_record_t temp = { 0 };
		rc = fds_record_open(&desc, &temp);
		APP_ERROR_CHECK(rc);
		memset(now_device_name, '\0', 20);
		memcpy(now_device_name, temp.p_data, (temp.p_header->length_words * 4));
		name_data = (uint8_t*)temp.p_data;
		NRF_LOG_INFO("%s /", name_data);
		rc = fds_record_close(&desc);
		APP_ERROR_CHECK(rc);
	}


}
static void interval_init(void)
{
	ret_code_t rc;
	fds_record_desc_t desc = { 0 };
	fds_find_token_t  tok = { 0 };
	uint8_t* interval_data;

	NRF_LOG_INFO("\ninterval_init\n");
	rc = fds_record_find(fds_record_interval.file_id, fds_record_interval.key, &desc, &tok);
	if (rc != FDS_SUCCESS)
	{
		NRF_LOG_INFO("interval not found in fds, set to default\n");
		now_record_interval_in_sec = 20;
		our_record_interval_data[0] = 0xBB;
		our_record_interval_data[1] = 0xB1;
		our_record_interval_data[2] = 0x00;
		our_record_interval_data[3] = 0x14;
		kls_fds_write(fds_record_interval.file_id, fds_record_interval.key, default_record_interval);
	}
	else
	{
		fds_flash_record_t temp = { 0 };
		rc = fds_record_open(&desc, &temp);
		APP_ERROR_CHECK(rc);
		interval_data = (uint8_t*)temp.p_data;
		for (uint8_t i = 0; i < 4; i++)
		{
			NRF_LOG_INFO("%x / ", interval_data[i]);
		}
		rc = fds_record_close(&desc);
		APP_ERROR_CHECK(rc);
		if (interval_data[0] == 0xBB && interval_data[1] == 0xB1)
		{
			if (interval_data[2] == 0x01) //second
			{
				now_record_interval_in_sec = 0;
				now_record_interval_in_sec = (uint32_t)interval_data[3];
				if (now_record_interval_in_sec == 0)
				{
					now_record_interval_in_sec = 20;
					our_record_interval_data[2] = 0x01;
					our_record_interval_data[3] = 0x14;
				}
				NRF_LOG_INFO("now_record_interval_in_sec = %d\n", now_record_interval_in_sec);
			}
			else if (interval_data[2] == 0x02) //minute
			{
				now_record_interval_in_sec = 0;
				now_record_interval_in_sec = ((interval_data[3] << 8) + interval_data[4]) * 60;
				if (now_record_interval_in_sec == 0)
				{
					now_record_interval_in_sec = 20;
					our_record_interval_data[2] = 0x01;
					our_record_interval_data[3] = 0x00;
					our_record_interval_data[4] = 0x14;
				}
				NRF_LOG_INFO("now_record_interval_in_sec = %d\n", now_record_interval_in_sec);
			}
			else
			{
				NRF_LOG_INFO("SEC/MIN error, set to default\n");
				now_record_interval_in_sec = 20;
				our_record_interval_data[2] = 0x00;
				our_record_interval_data[3] = 0x14;
				kls_fds_update(fds_record_interval.file_id, fds_record_interval.key, default_record_interval);

			}
		}
		else
		{
			NRF_LOG_INFO("interval not BBB1, set to default\n");
			now_record_interval_in_sec = 20;
			our_record_interval_data[0] = 0xBB;
			our_record_interval_data[1] = 0xB1;
			our_record_interval_data[2] = 0x00;
			our_record_interval_data[3] = 0x14;
			kls_fds_write(fds_record_interval.file_id, fds_record_interval.key, default_record_interval);
		}
	}



}
static void check_w25(void)
{
	uint8_t tmp[4];
	for (uint32_t i = FLASH_STA_ADDR; i < FLASH_END_ADDR; ++i)
	{
		W25Q80_Flash_BufferRead(tmp, i, 4);
		if (tmp[0] == 0xFF && tmp[1] == 0xFF && tmp[2] == 0xFF && tmp[3] == 0xFF)
		{
			gNowFlashAddr = i;
			break;
		}
	}
}
int main(void)
{
	ret_code_t rc;
	gHaveRecordTime = false;
	gHaveUartData = false;
	gSendDataStatus = 0;
	// Initialize.
	log_init();
	timer_init();
	twi_init();
	uart_init();
	power_management_init();
	ble_stack_init();
	(void)fds_register(fds_evt_handler);
	my_fds_info.busy = true;
	my_fds_info.gc = true;
	rc = fds_init();
	APP_ERROR_CHECK(rc);
	wait_for_fds_ready();
	fds_gc_action();
	pwd_init();
	device_name_init();
	interval_init();
	gap_params_init();
	gatt_init();

	services_init();
	advertising_init();

	conn_params_init();
	W25Q80_init();
	check_w25();
	advertising_start();
	nrf_gpio_cfg_output(LED);
	nrf_gpio_pin_clear(LED);
	nrf_gpio_cfg_input(BTN, GPIO_PIN_CNF_PULL_Pullup);
	app_timer_start(button_press_5s_timer_id, BUTTON_PRESS_5S_TIMER_INTERVAL, NULL);
	app_timer_start(m_one_sec_timer_id, ONE_SEC_TIMER_INTERVAL, NULL);

	// 	DS3231_set_data(20, 3, 30);
	// 	DS3231_set_time(9, 59, 30);
	// 	DS3231_read();
	// 	day_time_set(date_time[0], date_time[1], date_time[2], date_time[3], date_time[4], date_time[5]);

	NRF_LOG_INFO("Template example started\n");
	// Enter main loop.

	for (;;)
	{
		process();
		send_data_to_phone();
		idle_state_handle();
	}
}


