//FDS
#include "our_fds.h"
#include <stdint.h>
#include <string.h>
#include "fds.h"
#include "app_error.h"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

my_fds_info_t my_fds_info;

fds_data_t fdsData =
{
	.data = "BBBBBBBBBBBB"
};

//fds handle
void fds_evt_handler(fds_evt_t const* p_evt)
{
	switch (p_evt->id)
	{
	case FDS_EVT_INIT:
		if (p_evt->result == FDS_SUCCESS)
		{
			my_fds_info.busy = false;
		}
		break;
	case FDS_EVT_WRITE:
	{
		if (p_evt->result == FDS_SUCCESS)
		{
			my_fds_info.busy = false;
		}
	} break;

	case FDS_EVT_UPDATE:
	{
		if (p_evt->result == FDS_SUCCESS)
		{
			my_fds_info.busy = false;
		}
	} break;
	case FDS_EVT_GC:
	{
		if (p_evt->result == FDS_SUCCESS)
		{
			my_fds_info.busy = false;
		}
	} break;
	default:
		break;
	}
}

void kls_fds_write(uint16_t write_file_id, uint16_t write_record_key, uint8_t write_data[])
{
	ret_code_t rc;
	fds_record_desc_t desc = { 0 };
	fds_record_t record;

	record.file_id = write_file_id;
	record.key = write_record_key;
	record.data.p_data = write_data;
	record.data.length_words = (sizeof(fdsData) + 3) / sizeof(uint32_t);
	my_fds_info.busy = true;
	rc = fds_record_write(&desc, &record);
	if (rc != FDS_SUCCESS)
	{
		NRF_LOG_INFO("fds_record_write fail\n");
	}
	else
	{
		NRF_LOG_INFO("Writing Record ID = %d \r\n", desc.record_id);
	}
	wait_for_fds_ready();
}

void kls_fds_read(uint16_t read_file_id, uint16_t relevant_record_key, uint8_t read_data[])
{
	ret_code_t rc;
	fds_record_desc_t desc = { 0 };
	fds_find_token_t  tok = { 0 };
	memset(&tok, 0x00, sizeof(fds_find_token_t));
	uint8_t* data;
	rc = fds_record_find(read_file_id, relevant_record_key, &desc, &tok);
	if (rc == FDS_SUCCESS)
	{
		fds_flash_record_t temp = { 0 };
		rc = fds_record_open(&desc, &temp);
		APP_ERROR_CHECK(rc);

		data = (uint8_t*)temp.p_data;
		for (uint8_t i = 0; i < temp.p_header->length_words*4; i++)
		{
			read_data[i] = data[i];
			NRF_LOG_INFO("%x / ", read_data[i]);
		}
		NRF_LOG_INFO(" \n");
	
		rc = fds_record_close(&desc);
		APP_ERROR_CHECK(rc);
	}	
}

void kls_fds_update(uint16_t write_file_id, uint16_t write_record_key, uint8_t write_data[])
{
	ret_code_t rc;
	fds_record_desc_t desc = { 0 };
	fds_find_token_t  tok = { 0 };
	fds_record_t record;

	record.file_id = write_file_id;
	record.key = write_record_key;
	record.data.p_data = write_data;
	record.data.length_words = (sizeof(fdsData) + 3) / sizeof(uint32_t);
	memset(&tok, 0x00, sizeof(fds_find_token_t));
	my_fds_info.busy = true;
	rc = fds_record_find(write_file_id, write_record_key, &desc, &tok);
	if (rc == FDS_SUCCESS)
	{	
		rc = fds_record_update(&desc, &record);
		APP_ERROR_CHECK(rc);
		if (rc != FDS_SUCCESS)
		{
			NRF_LOG_INFO("fds_record_update fail\n");
		}
		else
		{
			NRF_LOG_INFO("Update ID = %d \r\n", desc.record_id);
		}
		wait_for_fds_ready();
	}
}
void fds_gc_action(void)
{
	ret_code_t rc;
	if (my_fds_info.gc == true)
	{
		my_fds_info.gc = false;
		my_fds_info.busy = true;
		rc = fds_gc();
		APP_ERROR_CHECK(rc);
		wait_for_fds_ready();
		NRF_LOG_INFO("FDS gc completed");
	}
}


void wait_for_fds_ready(void)
{
	while (my_fds_info.busy)
	{
		(void)sd_app_evt_wait();
	}
}
