#include <string.h>
#include "nrf_gpio.h"
#include "our_service.h"
#include "ble_srv_common.h"
#include "app_error.h"
#include "our_twi.h"
#include "our_fds.h"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"


uint8_t our_ble_event = 0x00;
uint8_t our_device_name_data[20] = { 0 };
uint8_t our_record_interval_data[4] = { 0 };
uint8_t our_confirm_password[20] = { 0 };
uint8_t our_change_password[20] = { 0 };
uint8_t test_oneshot_event;
// ALREADY_DONE_FOR_YOU: Declaration of a function that will take care of some housekeeping of ble connections related to our service and characteristic
void ble_our_service_on_ble_evt(ble_evt_t const* p_ble_evt, void* p_context)
{
	uint16_t z;
	uint8_t *temp;
	ble_os_t* p_our_service = (ble_os_t*)p_context;
	// OUR_JOB: Step 3.D Implement switch case handling BLE events related to our service. 
	switch (p_ble_evt->header.evt_id)
	{
	case BLE_GAP_EVT_CONNECTED:
		p_our_service->conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
		NRF_LOG_INFO("Connected to a previously bonded device.\n");
		break;
	case BLE_GAP_EVT_DISCONNECTED:
		p_our_service->conn_handle = BLE_CONN_HANDLE_INVALID;
		NRF_LOG_INFO("DISConnected to a previously bonded device.\n");
		break;
	case BLE_GATTS_EVT_WRITE:
		temp = p_ble_evt->evt.gatts_evt.params.write.data;
		if (p_ble_evt->evt.gatts_evt.params.write.handle == p_our_service->char_handles.value_handle)       //char_handles
		{
			NRF_LOG_INFO("uuid = %x ", p_ble_evt->evt.gatts_evt.params.write.uuid.uuid);
			z = p_ble_evt->evt.gatts_evt.params.write.len;
			NRF_LOG_INFO("data_len = %d ", z);			
			NRF_LOG_INFO("data = ");
			for (int x = 0; x < z; x++)
				NRF_LOG_INFO("%x / ", p_ble_evt->evt.gatts_evt.params.write.data[x]);
			NRF_LOG_INFO(" \n");

			if (temp[0] == 0xEE) {  //1.test
				if (temp[1] == 0xE1) {
					our_ble_event = 0xEE;
					test_oneshot_event = 0xE1;
				}
				else if (temp[1] == 0xE2) {
					our_ble_event = 0xEE;
					test_oneshot_event = 0xE2;
				}

			}

		}

		if (p_ble_evt->evt.gatts_evt.params.write.handle == p_our_service->pwd_char_handles.value_handle)   //pwd_char_handles
		{
			NRF_LOG_INFO("uuid = %x ", p_ble_evt->evt.gatts_evt.params.write.uuid.uuid);
			z = p_ble_evt->evt.gatts_evt.params.write.len;			
			NRF_LOG_INFO("data = ");
			for (int x = 0; x < z; x++)
				NRF_LOG_INFO("%x / ", p_ble_evt->evt.gatts_evt.params.write.data[x]);
			NRF_LOG_INFO(" \n");

			if (temp[0] == 0xAA && temp[1] == 0xA1)//1.confirm password
			{                       
				memcpy(our_confirm_password, temp, z);
				our_ble_event = 0xA1;
			}
			if (temp[0] == 0xAA && temp[1] == 0xA2)  //2.change password
			{                      
				memset(our_change_password, '\0', 20);
				memcpy(our_change_password, temp, z);
				our_ble_event = 0xA2;
			}
		}

		if (p_ble_evt->evt.gatts_evt.params.write.handle == p_our_service->record_char_handles.value_handle) //record_char_handles
		{
			NRF_LOG_INFO("uuid = %x ", p_ble_evt->evt.gatts_evt.params.write.uuid.uuid);
			z = p_ble_evt->evt.gatts_evt.params.write.len;		
			NRF_LOG_INFO("data = ");
			for (int x = 0; x < z; x++) {
				NRF_LOG_INFO("%x / ", p_ble_evt->evt.gatts_evt.params.write.data[x]);
			}
			NRF_LOG_INFO(" \n");
			if (temp[0] == 0xBB && temp[1] == 0xB1) {
				//do 1.set interval
				NRF_LOG_INFO("set interval!\n");
				memcpy(our_record_interval_data, temp, 5);
				//                  our_record_interval_data[0] = temp[2];
				//                  our_record_interval_data[1] = temp[3];
				//                  our_record_interval_data[2] = temp[4];
				our_ble_event = 0xB1;
			}
			if (temp[0] == 0xBB && temp[1] == 0xB2) {
				//do 2.start/stop record(download data when stop)
				NRF_LOG_INFO("start/stop record!\n");
				our_ble_event = 0xB2;
			}
			if (temp[0] == 0xBB && temp[1] == 0xB3) {
				//do 2.start/stop record(download data when stop)
				NRF_LOG_INFO("download record!\n");
				our_ble_event = 0xB3;
			}
			

		}

		if (p_ble_evt->evt.gatts_evt.params.write.handle == p_our_service->other_char_handles.value_handle) //other_char_handles
		{
			NRF_LOG_INFO("uuid = %x ", p_ble_evt->evt.gatts_evt.params.write.uuid.uuid);
			z = p_ble_evt->evt.gatts_evt.params.write.len;			
			NRF_LOG_INFO("data = ");
			for (int x = 0; x < z; x++) {
				NRF_LOG_INFO("%x / ", p_ble_evt->evt.gatts_evt.params.write.data[x]);
			}
			NRF_LOG_INFO(" \n");
			if (temp[0] == 0xCC && temp[1] == 0xC1) {
				//do 1.update time
				NRF_LOG_INFO("update time!\n");
				DS3231_set_data(temp[2], temp[3], temp[4]);
				DS3231_set_time(temp[5], temp[6], temp[7]);
			}
			if (temp[0] == 0xCC && temp[1] == 0xC2) {
				//do 2.show time by notification
				NRF_LOG_INFO("notify time\n");
				our_ble_event = 0xC2;
			}
			if (temp[0] == 0xCC && temp[1] == 0xC3) {
				//do 3.change device name
				memset(our_device_name_data, '\0', 20);
				memcpy(our_device_name_data, temp, z);
				NRF_LOG_INFO("data lenth = %d ,new device name data = ", temp[2]);
				for (int x = 0; x < z; x++)
					NRF_LOG_INFO("%x ", our_device_name_data[x]);
				NRF_LOG_INFO("\n");
				NRF_LOG_INFO("new device name char = %s\n ", our_device_name_data);
				our_ble_event = 0xC3;
			}

		}

		break;

	default:
		// No implementation needed.
		break;
	}

}

/**@brief Function for adding our new characterstic to "Our service" that we initiated in the previous tutorial.
 *
 * @param[in]   p_our_service        Our Service structure.
 *
 */
static uint32_t our_char_add(ble_os_t* p_our_service)
{
	uint32_t            err_code;
	ble_add_char_params_t add_char_params;
	memset(&add_char_params, 0, sizeof(add_char_params));
	add_char_params.uuid = BLE_UUID_OUR_CHARACTERISTC_UUID;
	add_char_params.uuid_type = BLE_UUID_TYPE_VENDOR_BEGIN;
	add_char_params.max_len = 50;
	add_char_params.init_len = 4;
	add_char_params.is_var_len = true;
	add_char_params.char_props.write = 1;
	add_char_params.char_props.read = 1;
	add_char_params.char_props.notify = 1;
	add_char_params.read_access = SEC_OPEN;
	add_char_params.write_access = SEC_OPEN;
	add_char_params.cccd_write_access = SEC_OPEN;

	uint8_t value[] = "our_char";
	add_char_params.init_len = strlen(value);
	add_char_params.p_init_value = value;
	err_code = characteristic_add(p_our_service->service_handle, &add_char_params, &p_our_service->char_handles);
	if (err_code != NRF_SUCCESS)
	{
		return err_code;
	}	
}
///////////////////////////////////////////////////////////////////////////////////////// pwd_char_add
static uint32_t pwd_char_add(ble_os_t* p_our_service)
{
	uint32_t            err_code;
	ble_add_char_params_t add_char_params;
	memset(&add_char_params, 0, sizeof(add_char_params));
	add_char_params.uuid = BLE_UUID_PWD_CHARACTERISTC_UUID;
	add_char_params.uuid_type = BLE_UUID_TYPE_VENDOR_BEGIN;
	add_char_params.max_len = 30;
	add_char_params.init_len = 4;
	add_char_params.is_var_len = true;
	add_char_params.char_props.write = 1;
	add_char_params.char_props.read = 1;
	add_char_params.char_props.notify = 1;
	add_char_params.read_access = SEC_OPEN;
	add_char_params.write_access = SEC_OPEN;
	add_char_params.cccd_write_access = SEC_OPEN;

	uint8_t value[] = "pwd_char";
	add_char_params.init_len = strlen(value);
	add_char_params.p_init_value = value;
	err_code = characteristic_add(p_our_service->service_handle, &add_char_params, &p_our_service->pwd_char_handles);
	if (err_code != NRF_SUCCESS)
	{
		return err_code;
	}	
}
///////////////////////////////////////////////////////////////////////////////////////// record_char_add
static uint32_t record_char_add(ble_os_t* p_our_service)
{
 	uint32_t            err_code;
	ble_add_char_params_t add_char_params;
	memset(&add_char_params, 0, sizeof(add_char_params));
	add_char_params.uuid = BLE_UUID_RECORD_CHARACTERISTC_UUID;
	add_char_params.uuid_type = BLE_UUID_TYPE_VENDOR_BEGIN;
	add_char_params.max_len = 20;
	add_char_params.init_len = sizeof(uint8_t);
	add_char_params.is_var_len = true;
	add_char_params.char_props.write = 1;
	add_char_params.char_props.read = 1;
	add_char_params.char_props.notify = 1;
	add_char_params.read_access = SEC_OPEN;
	add_char_params.write_access = SEC_OPEN;
	add_char_params.cccd_write_access = SEC_OPEN;
	uint8_t value[] = "record_char";	
	add_char_params.init_len= strlen(value);
	add_char_params.p_init_value = value;
	err_code = characteristic_add(p_our_service->service_handle, &add_char_params, &p_our_service->record_char_handles);
	if (err_code != NRF_SUCCESS)
	{
		return err_code;
	}
}
///////////////////////////////////////////////////////////////////////////////////////// other_char_add
static uint32_t other_char_add(ble_os_t* p_our_service)
{
	uint32_t            err_code;
	ble_add_char_params_t add_char_params;
	memset(&add_char_params, 0, sizeof(add_char_params));
	add_char_params.uuid = BLE_UUID_OTHER_CHARACTERISTC_UUID;
	add_char_params.uuid_type = BLE_UUID_TYPE_VENDOR_BEGIN;
	add_char_params.max_len = 20;
	add_char_params.init_len = 4;
	add_char_params.is_var_len = true;
	add_char_params.char_props.write = 1;
	add_char_params.char_props.read = 1;
	add_char_params.char_props.notify = 1;
	add_char_params.read_access = SEC_OPEN;
	add_char_params.write_access = SEC_OPEN;
	add_char_params.cccd_write_access = SEC_OPEN;

	uint8_t value[] = "other_char";
	add_char_params.init_len = strlen(value);
	add_char_params.p_init_value = value;
	err_code = characteristic_add(p_our_service->service_handle, &add_char_params, &p_our_service->other_char_handles);
	if (err_code != NRF_SUCCESS)
	{
		return err_code;
	}

}
/**@brief Function for initiating our new service.
 *
 * @param[in]   p_our_service        Our Service structure.
 *
 */
void our_service_init(ble_os_t* p_our_service)
{
	uint32_t   err_code; // Variable to hold return codes from library and softdevice functions

	// FROM_SERVICE_TUTORIAL: Declare 16-bit service and 128-bit base UUIDs and add them to the BLE stack
	ble_uuid_t        service_uuid;
	ble_uuid128_t     base_uuid = BLE_UUID_OUR_BASE_UUID;
	service_uuid.uuid = BLE_UUID_OUR_SERVICE_UUID;
	service_uuid.type = BLE_UUID_TYPE_VENDOR_BEGIN;
	err_code = sd_ble_uuid_vs_add(&base_uuid, &service_uuid.type);
	APP_ERROR_CHECK(err_code);

	// OUR_JOB: Step 3.B, Set our service connection handle to default value. I.e. an invalid handle since we are not yet in a connection.
	p_our_service->conn_handle = BLE_CONN_HANDLE_INVALID;

	// FROM_SERVICE_TUTORIAL: Add our service
	err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY,
		&service_uuid,
		&p_our_service->service_handle);

	APP_ERROR_CHECK(err_code);

	// OUR_JOB: Call the function our_char_add() to add our new characteristic to the service. 
	our_char_add(p_our_service);
	pwd_char_add(p_our_service);
	record_char_add(p_our_service);
	other_char_add(p_our_service);
}

// ALREADY_DONE_FOR_YOU: Function to be called when updating characteristic value
void our_temperature_characteristic_update(ble_os_t* p_our_service, int32_t* temperature_value)
{
	// OUR_JOB: Step 3.E, Update characteristic value
	if (p_our_service->conn_handle != BLE_CONN_HANDLE_INVALID)
	{
		uint16_t               len = 50;
		uint8_t               test[] = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa012345678911234567892123456789";
		ble_gatts_hvx_params_t hvx_params;
		memset(&hvx_params, 0, sizeof(hvx_params));

		hvx_params.handle = p_our_service->char_handles.value_handle;
		hvx_params.type = BLE_GATT_HVX_NOTIFICATION;
		hvx_params.offset = 0;
		hvx_params.p_len = &len;
		hvx_params.p_data = (uint8_t*)test;//(uint8_t*)temperature_value;  

		sd_ble_gatts_hvx(p_our_service->conn_handle, &hvx_params);
	}

}

void update_attribute(ble_os_t* p_our_service, uint8_t model, uint8_t version, bool record_state, uint8_t interval[], uint32_t flashNum)
{
	uint32_t   err_code;
	ble_gatts_value_t gatts_value;
	uint8_t update_value[10];
	update_value[0] = model;
	update_value[1] = version;
	update_value[2] = record_state;
	update_value[3] = interval[0];
	update_value[4] = interval[1];
	update_value[5] = interval[2];
	update_value[6] = flashNum >> 24 & 0xff;
	update_value[7] = flashNum >> 16 & 0xff;
	update_value[8] = flashNum >> 8 & 0xff;
	update_value[9] = flashNum & 0xff;

	// Initialize value struct.
	memset(&gatts_value, 0, sizeof(gatts_value));

	gatts_value.len = 10;
	gatts_value.offset = 0;
	gatts_value.p_value = update_value;

	// Update database. // Need to understand about it
		// Set a new value of the custom_value which will be sent to client.
	err_code = sd_ble_gatts_value_set(p_our_service->conn_handle,
		p_our_service->record_char_handles.value_handle,
		&gatts_value);

}
