
#ifndef OUR_SERVICE_H__
#define OUR_SERVICE_H__

#include <stdint.h>
#include "ble.h"
#include "ble_srv_common.h"

// FROM_SERVICE_TUTORIAL: Defining 16-bit service and 128-bit base UUIDs
#define BLE_UUID_OUR_BASE_UUID              {{0x23, 0xD1, 0x13, 0xEF, 0x5F, 0x78, 0x23, 0x15, 0xDE, 0xEF, 0x12, 0x12, 0x00, 0x00, 0x00, 0x00}} // 128-bit base UUID
#define BLE_UUID_OUR_SERVICE_UUID                0xF00D // Just a random, but recognizable value

// ALREADY_DONE_FOR_YOU: Defining 16-bit characteristic UUID
#define BLE_UUID_OUR_CHARACTERISTC_UUID          0xBEEF // for f249 notification
#define BLE_UUID_PWD_CHARACTERISTC_UUID          0xAAAA // for password 1.confirm password, 2.change password
#define BLE_UUID_RECORD_CHARACTERISTC_UUID       0xBBBB // for record   1.set interval, 2.start/stop record, 3.download data  read: internal, mode, download or not     
#define BLE_UUID_OTHER_CHARACTERISTC_UUID        0xCCCC // for others   1.update time

//ble_os_t m_our_service;
// This structure contains various status information for our service. 
// The name is based on the naming convention used in Nordics SDKs. 
// 'bleindicates that it is a Bluetooth Low Energy relevant structure and 
// osis short for Our Service). 
typedef struct
{
    uint16_t                    conn_handle;    /**< Handle of the current connection (as provided by the BLE stack, is BLE_CONN_HANDLE_INVALID if not in a connection).*/
    uint16_t                    service_handle; /**< Handle of Our Service (as provided by the BLE stack). */
    // OUR_JOB: Step 2.D, Add handles for the characteristic attributes to our struct
    ble_gatts_char_handles_t    char_handles;
    ble_gatts_char_handles_t    pwd_char_handles;
    ble_gatts_char_handles_t    record_char_handles;
    ble_gatts_char_handles_t    other_char_handles;
}ble_os_t;

/**@brief Function for handling BLE Stack events related to our service and characteristic.
 *
 * @details Handles all events from the BLE stack of interest to Our Service.
 *
 * @param[in]   p_our_service       Our Service structure.
 * @param[in]   p_ble_evt  Event received from the BLE stack.
 */
void ble_our_service_on_ble_evt(ble_evt_t const * p_ble_evt, void * p_context);

/**@brief Function for initializing our new service.
 *
 * @param[in]   p_our_service       Pointer to Our Service structure.
 */
void our_service_init(ble_os_t * p_our_service);

/**@brief Function for updating and sending new characteristic values
 *
 * @details The application calls this function whenever our timer_timeout_handler triggers
 *
 * @param[in]   p_our_service                     Our Service structure.
 * @param[in]   characteristic_value     New characteristic value.
 */
void our_temperature_characteristic_update(ble_os_t *p_our_service, int32_t *temperature_value);
void update_attribute(ble_os_t * p_our_service, uint8_t model, uint8_t version, bool record_state, uint8_t interval[], uint32_t flashNum);

extern uint8_t our_record_interval_data[4];
extern uint8_t test_oneshot_event;
extern uint8_t our_ble_event;
extern uint8_t our_confirm_password[20];
extern uint8_t our_change_password[20];
extern uint8_t our_device_name_data[20];
#endif  /* _ OUR_SERVICE_H__ */
