#include "our_spi.h"
#include "nrf_drv_spi.h"
#include "app_util_platform.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "boards.h"
#include "app_error.h"
#include <stdint.h>
#include <string.h>
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

uint8_t   SPI_Tx_Buf[SPI_BUFSIZE];  //
uint8_t   SPI_Rx_Buf[SPI_BUFSIZE];  //收
volatile  uint8_t   SPIReadLength, SPIWriteLength;

void spi_event_handler(nrf_drv_spi_evt_t const * p_event,
                       void *                    p_context)
{
    spi_xfer_done = true;
}

void W25Q80_write_reg(int data)
{
    spi_xfer_done = false;
    SPIWriteLength = 1;
    SPIReadLength = 0;
    SPI_Tx_Buf[0] = data;  
    APP_ERROR_CHECK(nrf_drv_spi_transfer(&spi, SPI_Tx_Buf, SPIWriteLength, SPI_Rx_Buf, SPIReadLength));
    while(spi_xfer_done == false);
}

uint8_t W25Q80_read_reg(int reg)
{
    spi_xfer_done = false;
    SPI_Tx_Buf[0] = reg;
    APP_ERROR_CHECK(nrf_drv_spi_transfer(&spi, SPI_Tx_Buf, 0, SPI_Rx_Buf,1));
    while(spi_xfer_done == false);
    return SPI_Rx_Buf[0];
}

uint32_t W25Q80_ReadID(void)
{
    uint32_t temp = 0,temp0 = 0,temp1 = 0,temp2 = 0;
    nrf_gpio_pin_clear(SPI_SS_PIN);  
    W25Q80_write_reg(W25X_JedecDeviceID);
    temp0 = W25Q80_read_reg(0XFF);
    temp1 = W25Q80_read_reg(0XFF);
    temp2 = W25Q80_read_reg(0XFF);
    nrf_gpio_pin_set(SPI_SS_PIN); 
    temp = (temp0 << 16)| (temp1 << 8) | temp2;
    return temp;
}

void W25Q80_WriteEnable()
{
    nrf_gpio_pin_clear(SPI_SS_PIN); 
    W25Q80_write_reg(W25X_WriteEnable);
    nrf_gpio_pin_set(SPI_SS_PIN);  
}

void W25Q80_WaitForWriteEnd()
{
    unsigned char FLASH_Status = 0;
    nrf_gpio_pin_clear(SPI_SS_PIN); 
    W25Q80_write_reg(W25X_ReadStatusReg); 
    do
    {
        FLASH_Status = W25Q80_read_reg(Dummy_Byte);
    }
    while((WIP_Flag & FLASH_Status) == 1);
    nrf_gpio_pin_set(SPI_SS_PIN); 
    
}

void W25Q80_FLASH_SectorErase(uint32_t SectorAddr)
{
    W25Q80_WriteEnable(); 
    W25Q80_WaitForWriteEnd(); 
    nrf_gpio_pin_clear(SPI_SS_PIN); 
    W25Q80_write_reg(W25X_SectorErase); 
    W25Q80_write_reg((SectorAddr & 0XFF0000) >> 16);
    W25Q80_write_reg((SectorAddr & 0XFF00) >> 8);
    W25Q80_write_reg(SectorAddr & 0XFF);
    nrf_gpio_pin_set(SPI_SS_PIN);  
    W25Q80_WaitForWriteEnd();
}

void W25Q80_FLASH_PageWrite(uint8_t* pBuffer, uint32_t WriteAddr, uint16_t NumByteToWrite)
{
    W25Q80_WriteEnable(); 
    nrf_gpio_pin_clear(SPI_SS_PIN);  
    W25Q80_write_reg(W25X_PageProgram); 
    W25Q80_write_reg((WriteAddr & 0XFF0000) >> 16);
    W25Q80_write_reg((WriteAddr & 0XFF00) >> 8);
    W25Q80_write_reg(WriteAddr & 0XFF);
    if(NumByteToWrite > 256)
    {
        NRF_LOG_INFO("write too large!\r\n");
        return ;
    }
    while(NumByteToWrite--)
    {
        W25Q80_write_reg(*pBuffer);
        pBuffer++;
    }
    nrf_gpio_pin_set(SPI_SS_PIN); 
    W25Q80_WaitForWriteEnd(); 
}

void W25Q80_Flash_BufferRead(uint8_t* pBuffer, uint32_t ReadAddr, uint16_t NumByteToRead)
{
    nrf_gpio_pin_clear(SPI_SS_PIN);
    W25Q80_write_reg(W25X_ReadData); 
    W25Q80_write_reg((ReadAddr & 0XFF0000) >> 16);
    W25Q80_write_reg((ReadAddr & 0XFF00) >> 8);
    W25Q80_write_reg(ReadAddr & 0XFF);
    while(NumByteToRead--)
    {
        *pBuffer = W25Q80_read_reg(Dummy_Byte);
        pBuffer++;
    }
    nrf_gpio_pin_set(SPI_SS_PIN);
}

void W25Q80_Chip_Erase()
{
    W25Q80_WriteEnable(); 
    nrf_gpio_pin_clear(SPI_SS_PIN); 
    W25Q80_write_reg(W25X_ChipErase);  
    nrf_gpio_pin_set(SPI_SS_PIN);  
    W25Q80_WaitForWriteEnd();  
}
void W25Q80_init()
{
    nrf_drv_spi_config_t spi_config = NRF_DRV_SPI_DEFAULT_CONFIG;
//    spi_config.ss_pin   = SPI_SS_PIN;  //SS引禁用
    spi_config.miso_pin = SPI_MISO_PIN;
    spi_config.mosi_pin = SPI_MOSI_PIN;
    spi_config.sck_pin  = SPI_SCK_PIN;
    spi_config.irq_priority = 2,
    nrf_gpio_cfg_output(SPI_SS_PIN);
    nrf_gpio_pin_clear(SPI_SS_PIN);  
    APP_ERROR_CHECK(nrf_drv_spi_init(&spi, &spi_config, spi_event_handler, NULL));

}